@@ -76,24 +76,19 @@
 #define _snd_seq_query_subscribe sndrv_seq_query_subs
 #define _snd_seq_client_pool sndrv_seq_client_pool
 #define _snd_seq_remove_events sndrv_seq_remove_events
-#define _snd_instr_header sndrv_seq_instr_header
 
 #define sndrv_seq_addr	snd_seq_addr
 #define sndrv_seq_tick_time_t	snd_seq_tick_time_t
 #define sndrv_seq_real_time	snd_seq_real_time
 #define sndrv_seq_timestamp	snd_seq_timestamp
-#define sndrv_seq_instr		snd_seq_instr
 #define sndrv_seq_event		snd_seq_event
-#define sndrv_seq_instr_cluster_t	snd_seq_instr_cluster_t
 
 #if 0
 typedef struct sndrv_seq_addr snd_seq_addr_t;
 #define snd_seq_tick_time_t sndrv_seq_tick_time_t
 typedef struct sndrv_seq_real_time snd_seq_real_time_t;
 typedef union sndrv_seq_timestamp snd_seq_timestamp_t;
-typedef struct sndrv_seq_instr snd_seq_instr_t;
 typedef struct sndrv_seq_event snd_seq_event_t;
-#define snd_seq_instr_cluster_t sndrv_seq_instr_cluster_t
 #endif
 
 #define _snd_timer_id sndrv_timer_id
@@ -128,8 +123,6 @@ typedef struct sndrv_seq_event snd_seq_event_t;
 #include <sound/asequencer.h>
 #include "seqmid.h"
 #include "seq_midi_event.h"
-#include "conv.h"
-#include "instr.h"
 #include "list.h"
 
 #if __BYTE_ORDER == __LITTLE_ENDIAN
@@ -254,6 +247,20 @@ static inline int snd_open_device(const char *filename, int fmode)
 #define snd_open_device(filename, fmode) open(filename, fmode);
 #endif
 
+/* make local functions really local */
+#define snd_dlobj_cache_lookup \
+	snd1_dlobj_cache_lookup
+#define snd_dlobj_cache_add \
+	snd1_dlobj_cache_add
+#define snd_dlobj_cache_cleanup \
+	snd1_dlobj_cache_cleanup
+#define snd_config_set_hop \
+	snd1_config_set_hop
+#define snd_config_check_hop \
+	snd1_config_check_hop
+#define snd_config_search_alias_hooks \
+	snd1_config_search_alias_hooks
+
 /* dlobj cache */
 void *snd_dlobj_cache_lookup(const char *name);
 int snd_dlobj_cache_add(const char *name, void *dlobj, void *open_func);
