@@ -156,95 +156,124 @@ static void mix_areas(snd_pcm_direct_t *dmix,
 		      snd_pcm_uframes_t dst_ofs,
 		      snd_pcm_uframes_t size)
 {
-	volatile signed int *sum;
 	unsigned int src_step, dst_step;
-	unsigned int chn, dchn, channels;
+	unsigned int chn, dchn, channels, sample_size;
+	mix_areas_t *do_mix_areas;
 	
 	channels = dmix->channels;
-	if (dmix->shmptr->s.format == SND_PCM_FORMAT_S16_LE ||
-	    dmix->shmptr->s.format == SND_PCM_FORMAT_S16_BE) {
-		signed short *src;
-		volatile signed short *dst;
-		if (dmix->interleaved) {
-			/*
-			 * process all areas in one loop
-			 * it optimizes the memory accesses for this case
-			 */
-			dmix->u.dmix.mix_areas1(size * channels,
-					 ((signed short *)dst_areas[0].addr) + (dst_ofs * channels),
-					 ((signed short *)src_areas[0].addr) + (src_ofs * channels),
-					 dmix->u.dmix.sum_buffer + (dst_ofs * channels),
-					 sizeof(signed short),
-					 sizeof(signed short),
-					 sizeof(signed int));
-			return;
-		}
-		for (chn = 0; chn < channels; chn++) {
-			dchn = dmix->bindings ? dmix->bindings[chn] : chn;
-			if (dchn >= dmix->shmptr->s.channels)
-				continue;
-			src_step = src_areas[chn].step / 8;
-			dst_step = dst_areas[dchn].step / 8;
-			src = (signed short *)(((char *)src_areas[chn].addr + src_areas[chn].first / 8) + (src_ofs * src_step));
-			dst = (signed short *)(((char *)dst_areas[dchn].addr + dst_areas[dchn].first / 8) + (dst_ofs * dst_step));
-			sum = dmix->u.dmix.sum_buffer + channels * dst_ofs + chn;
-			dmix->u.dmix.mix_areas1(size, dst, src, sum, dst_step, src_step, channels * sizeof(signed int));
-		}
-	} else if (dmix->shmptr->s.format == SND_PCM_FORMAT_S32_LE ||
-		   dmix->shmptr->s.format == SND_PCM_FORMAT_S32_BE) {
-		signed int *src;
-		volatile signed int *dst;
-		if (dmix->interleaved) {
-			/*
-			 * process all areas in one loop
-			 * it optimizes the memory accesses for this case
-			 */
-			dmix->u.dmix.mix_areas2(size * channels,
-					 ((signed int *)dst_areas[0].addr) + (dst_ofs * channels),
-					 ((signed int *)src_areas[0].addr) + (src_ofs * channels),
-					 dmix->u.dmix.sum_buffer + (dst_ofs * channels),
-					 sizeof(signed int),
-					 sizeof(signed int),
-					 sizeof(signed int));
-			return;
-		}
-		for (chn = 0; chn < channels; chn++) {
-			dchn = dmix->bindings ? dmix->bindings[chn] : chn;
-			if (dchn >= dmix->shmptr->s.channels)
-				continue;
-			src_step = src_areas[chn].step / 8;
-			dst_step = dst_areas[dchn].step / 8;
-			src = (signed int *)(((char *)src_areas[chn].addr + src_areas[chn].first / 8) + (src_ofs * src_step));
-			dst = (signed int *)(((char *)dst_areas[dchn].addr + dst_areas[dchn].first / 8) + (dst_ofs * dst_step));
-			sum = dmix->u.dmix.sum_buffer + channels * dst_ofs + chn;
-			dmix->u.dmix.mix_areas2(size, dst, src, sum, dst_step, src_step, channels * sizeof(signed int));
-		}
-	} else { /* SND_PCM_FORMAT_S24_3LE */
-		unsigned char *src;
-		volatile unsigned char *dst;
-		if (dmix->interleaved) {
-			/*
-			 * process all areas in one loop
-			 * it optimizes the memory accesses for this case
-			 */
-			dmix->u.dmix.mix_areas3(size * channels,
-					((unsigned char *)dst_areas[0].addr) + 3 * dst_ofs * channels,
-					((unsigned char *)src_areas[0].addr) + 3 * src_ofs * channels,
-					dmix->u.dmix.sum_buffer + (dst_ofs * channels),
-					3, 3, sizeof(signed int));
-			return;
-		}
-		for (chn = 0; chn < channels; chn++) {
-			dchn = dmix->bindings ? dmix->bindings[chn] : chn;
-			if (dchn >= dmix->shmptr->s.channels)
-				continue;
-			src_step = src_areas[chn].step / 8;
-			dst_step = dst_areas[dchn].step / 8;
-			src = (unsigned char *)(((char *)src_areas[chn].addr + src_areas[chn].first / 8) + (src_ofs * src_step));
-			dst = (unsigned char *)(((char *)dst_areas[dchn].addr + dst_areas[dchn].first / 8) + (dst_ofs * dst_step));
-			sum = dmix->u.dmix.sum_buffer + channels * dst_ofs + chn;
-			dmix->u.dmix.mix_areas3(size, dst, src, sum, dst_step, src_step, channels * sizeof(signed int));
-		}
+	switch (dmix->shmptr->s.format) {
+	case SND_PCM_FORMAT_S16_LE:
+	case SND_PCM_FORMAT_S16_BE:
+		sample_size = 2;
+		do_mix_areas = (mix_areas_t *)dmix->u.dmix.mix_areas_16;
+		break;
+	case SND_PCM_FORMAT_S32_LE:
+	case SND_PCM_FORMAT_S32_BE:
+		sample_size = 4;
+		do_mix_areas = (mix_areas_t *)dmix->u.dmix.mix_areas_32;
+		break;
+	case SND_PCM_FORMAT_S24_3LE:
+		sample_size = 3;
+		do_mix_areas = (mix_areas_t *)dmix->u.dmix.mix_areas_24;
+		break;
+	case SND_PCM_FORMAT_U8:
+		sample_size = 1;
+		do_mix_areas = (mix_areas_t *)dmix->u.dmix.mix_areas_u8;
+		break;
+	default:
+		return;
+	}
+	if (dmix->interleaved) {
+		/*
+		 * process all areas in one loop
+		 * it optimizes the memory accesses for this case
+		 */
+		do_mix_areas(size * channels,
+			     (unsigned char *)dst_areas[0].addr + sample_size * dst_ofs * channels,
+			     (unsigned char *)src_areas[0].addr + sample_size * src_ofs * channels,
+			     dmix->u.dmix.sum_buffer + dst_ofs * channels,
+			     sample_size,
+			     sample_size,
+			     sizeof(signed int));
+		return;
+	}
+	for (chn = 0; chn < channels; chn++) {
+		dchn = dmix->bindings ? dmix->bindings[chn] : chn;
+		if (dchn >= dmix->shmptr->s.channels)
+			continue;
+		src_step = src_areas[chn].step / 8;
+		dst_step = dst_areas[dchn].step / 8;
+		do_mix_areas(size,
+			     ((unsigned char *)dst_areas[dchn].addr + dst_areas[dchn].first / 8) + dst_ofs * dst_step,
+			     ((unsigned char *)src_areas[chn].addr + src_areas[chn].first / 8) + src_ofs * src_step,
+			     dmix->u.dmix.sum_buffer + channels * dst_ofs + chn,
+			     dst_step,
+			     src_step,
+			     channels * sizeof(signed int));
+	}
+}
+
+static void remix_areas(snd_pcm_direct_t *dmix,
+			const snd_pcm_channel_area_t *src_areas,
+			const snd_pcm_channel_area_t *dst_areas,
+			snd_pcm_uframes_t src_ofs,
+			snd_pcm_uframes_t dst_ofs,
+			snd_pcm_uframes_t size)
+{
+	unsigned int src_step, dst_step;
+	unsigned int chn, dchn, channels, sample_size;
+	mix_areas_t *do_remix_areas;
+	
+	channels = dmix->channels;
+	switch (dmix->shmptr->s.format) {
+	case SND_PCM_FORMAT_S16_LE:
+	case SND_PCM_FORMAT_S16_BE:
+		sample_size = 2;
+		do_remix_areas = (mix_areas_t *)dmix->u.dmix.remix_areas_16;
+		break;
+	case SND_PCM_FORMAT_S32_LE:
+	case SND_PCM_FORMAT_S32_BE:
+		sample_size = 4;
+		do_remix_areas = (mix_areas_t *)dmix->u.dmix.remix_areas_32;
+		break;
+	case SND_PCM_FORMAT_S24_3LE:
+		sample_size = 3;
+		do_remix_areas = (mix_areas_t *)dmix->u.dmix.remix_areas_24;
+		break;
+	case SND_PCM_FORMAT_U8:
+		sample_size = 1;
+		do_remix_areas = (mix_areas_t *)dmix->u.dmix.remix_areas_u8;
+		break;
+	default:
+		return;
+	}
+	if (dmix->interleaved) {
+		/*
+		 * process all areas in one loop
+		 * it optimizes the memory accesses for this case
+		 */
+		do_remix_areas(size * channels,
+			       (unsigned char *)dst_areas[0].addr + sample_size * dst_ofs * channels,
+			       (unsigned char *)src_areas[0].addr + sample_size * src_ofs * channels,
+			       dmix->u.dmix.sum_buffer + dst_ofs * channels,
+			       sample_size,
+			       sample_size,
+			       sizeof(signed int));
+		return;
+	}
+	for (chn = 0; chn < channels; chn++) {
+		dchn = dmix->bindings ? dmix->bindings[chn] : chn;
+		if (dchn >= dmix->shmptr->s.channels)
+			continue;
+		src_step = src_areas[chn].step / 8;
+		dst_step = dst_areas[dchn].step / 8;
+		do_remix_areas(size,
+			       ((unsigned char *)dst_areas[dchn].addr + dst_areas[dchn].first / 8) + dst_ofs * dst_step,
+			       ((unsigned char *)src_areas[chn].addr + src_areas[chn].first / 8) + src_ofs * src_step,
+			       dmix->u.dmix.sum_buffer + channels * dst_ofs + chn,
+			       dst_step,
+			       src_step,
+			       channels * sizeof(signed int));
 	}
 }
 
@@ -269,7 +298,7 @@ static void snd_pcm_dmix_sync_area(snd_pcm_t *pcm)
 {
 	snd_pcm_direct_t *dmix = pcm->private_data;
 	snd_pcm_uframes_t slave_hw_ptr, slave_appl_ptr, slave_size;
-	snd_pcm_uframes_t appl_ptr, size;
+	snd_pcm_uframes_t appl_ptr, size, transfer;
 	const snd_pcm_channel_area_t *src_areas, *dst_areas;
 	
 	/* calculate the size to transfer */
@@ -282,6 +311,27 @@ static void snd_pcm_dmix_sync_area(snd_pcm_t *pcm)
 	if (size >= pcm->boundary / 2)
 		size = pcm->boundary - size;
 
+	/* the slave_app_ptr can be far behind the slave_hw_ptr */
+	/* reduce mixing and errors here - just skip not catched writes */
+	if (dmix->slave_hw_ptr <= dmix->slave_appl_ptr)
+		slave_size = dmix->slave_appl_ptr - dmix->slave_hw_ptr;
+	else
+		slave_size = dmix->slave_appl_ptr + (dmix->slave_boundary - dmix->slave_hw_ptr);
+	if (slave_size > dmix->slave_buffer_size) {
+		transfer = dmix->slave_buffer_size - slave_size;
+		if (transfer > size)
+			transfer = size;
+		dmix->last_appl_ptr += transfer;
+		dmix->last_appl_ptr %= pcm->boundary;
+		dmix->slave_appl_ptr += transfer;
+		dmix->slave_appl_ptr %= dmix->slave_boundary;
+		size = dmix->appl_ptr - dmix->last_appl_ptr;
+		if (! size)
+			return;
+		if (size >= pcm->boundary / 2)
+			size = pcm->boundary - size;
+	}
+
 	/* check the available size in the slave PCM buffer */
 	slave_hw_ptr = dmix->slave_hw_ptr;
 	/* don't write on the last active period - this area may be cleared
@@ -311,7 +361,7 @@ static void snd_pcm_dmix_sync_area(snd_pcm_t *pcm)
 	dmix->slave_appl_ptr %= dmix->slave_boundary;
 	dmix_down_sem(dmix);
 	for (;;) {
-		snd_pcm_uframes_t transfer = size;
+		transfer = size;
 		if (appl_ptr + transfer > pcm->buffer_size)
 			transfer = pcm->buffer_size - appl_ptr;
 		if (slave_appl_ptr + transfer > dmix->slave_buffer_size)
@@ -367,11 +417,8 @@ static int snd_pcm_dmix_sync_ptr(snd_pcm_t *pcm)
 	if (avail > dmix->avail_max)
 		dmix->avail_max = avail;
 	if (avail >= pcm->stop_threshold) {
-		struct timeval tv;
 		snd_timer_stop(dmix->timer);
-		gettimeofday(&tv, 0);
-		dmix->trigger_tstamp.tv_sec = tv.tv_sec;
-		dmix->trigger_tstamp.tv_nsec = tv.tv_usec * 1000L;
+		gettimestamp(&dmix->trigger_tstamp, pcm->monotonic);
 		if (dmix->state == SND_PCM_STATE_RUNNING) {
 			dmix->state = SND_PCM_STATE_XRUN;
 			return -EPIPE;
@@ -420,7 +467,7 @@ static int snd_pcm_dmix_status(snd_pcm_t *pcm, snd_pcm_status_t * status)
 	memset(status, 0, sizeof(*status));
 	status->state = snd_pcm_dmix_state(pcm);
 	status->trigger_tstamp = dmix->trigger_tstamp;
-	status->tstamp = snd_pcm_hw_fast_tstamp(dmix->spcm);
+	gettimestamp(&status->tstamp, pcm->monotonic);
 	status->avail = snd_pcm_mmap_playback_avail(pcm);
 	status->avail_max = status->avail > dmix->avail_max ? status->avail : dmix->avail_max;
 	dmix->avail_max = 0;
@@ -525,7 +572,6 @@ static int snd_pcm_dmix_start(snd_pcm_t *pcm)
 {
 	snd_pcm_direct_t *dmix = pcm->private_data;
 	snd_pcm_sframes_t avail;
-	struct timeval tv;
 	int err;
 	
 	if (dmix->state != SND_PCM_STATE_PREPARED)
@@ -540,9 +586,7 @@ static int snd_pcm_dmix_start(snd_pcm_t *pcm)
 			return err;
 		snd_pcm_dmix_sync_area(pcm);
 	}
-	gettimeofday(&tv, 0);
-	dmix->trigger_tstamp.tv_sec = tv.tv_sec;
-	dmix->trigger_tstamp.tv_nsec = tv.tv_usec * 1000L;
+	gettimestamp(&dmix->trigger_tstamp, pcm->monotonic);
 	return 0;
 }
 
@@ -605,15 +649,88 @@ static int snd_pcm_dmix_pause(snd_pcm_t *pcm ATTRIBUTE_UNUSED, int enable ATTRIB
 	return -EIO;
 }
 
-static snd_pcm_sframes_t snd_pcm_dmix_rewind(snd_pcm_t *pcm ATTRIBUTE_UNUSED, snd_pcm_uframes_t frames ATTRIBUTE_UNUSED)
+static snd_pcm_sframes_t snd_pcm_dmix_rewindable(snd_pcm_t *pcm)
 {
-#if 0
-	/* FIXME: substract samples from the mix ring buffer, too? */
+	return snd_pcm_mmap_hw_avail(pcm);
+}
+
+static snd_pcm_sframes_t snd_pcm_dmix_rewind(snd_pcm_t *pcm, snd_pcm_uframes_t frames)
+{
+	snd_pcm_direct_t *dmix = pcm->private_data;
+	snd_pcm_uframes_t slave_appl_ptr, slave_size;
+	snd_pcm_uframes_t appl_ptr, size, transfer, result;
+	const snd_pcm_channel_area_t *src_areas, *dst_areas;
+
+	if (dmix->state == SND_PCM_STATE_RUNNING ||
+	    dmix->state == SND_PCM_STATE_DRAINING)
+	    	return snd_pcm_dmix_hwsync(pcm);
+
+	if (dmix->last_appl_ptr < dmix->appl_ptr)
+		size = dmix->appl_ptr - dmix->last_appl_ptr;
+	else
+		size = dmix->appl_ptr + (pcm->boundary - dmix->last_appl_ptr);
+	if (frames < size)
+		size = frames;
+	snd_pcm_mmap_appl_backward(pcm, size);
+	frames -= size;
+	if (!frames)
+		return size;
+	result = size;
+
+	if (dmix->hw_ptr < dmix->appl_ptr)
+		size = dmix->appl_ptr - dmix->hw_ptr;
+	else
+		size = dmix->appl_ptr + (pcm->boundary - dmix->hw_ptr);
+	if (size > frames)
+		size = frames;
+	if (dmix->slave_hw_ptr < dmix->slave_appl_ptr)
+		slave_size = dmix->slave_appl_ptr - dmix->slave_hw_ptr;
+	else
+		slave_size = dmix->slave_appl_ptr + (pcm->boundary - dmix->slave_hw_ptr);
+	if (slave_size < size)
+		size = slave_size;
+	frames -= size;
+	result += size;
+		
+	/* add sample areas here */
+	src_areas = snd_pcm_mmap_areas(pcm);
+	dst_areas = snd_pcm_mmap_areas(dmix->spcm);
+	dmix->last_appl_ptr -= size;
+	dmix->last_appl_ptr %= pcm->boundary;
+	appl_ptr = dmix->last_appl_ptr % pcm->buffer_size;
+	dmix->slave_appl_ptr -= size;
+	dmix->slave_appl_ptr %= dmix->slave_boundary;
+	slave_appl_ptr = dmix->slave_appl_ptr % dmix->slave_buffer_size;
+	dmix_down_sem(dmix);
+	for (;;) {
+		transfer = size;
+		if (appl_ptr + transfer > pcm->buffer_size)
+			transfer = pcm->buffer_size - appl_ptr;
+		if (slave_appl_ptr + transfer > dmix->slave_buffer_size)
+			transfer = dmix->slave_buffer_size - slave_appl_ptr;
+		remix_areas(dmix, src_areas, dst_areas, appl_ptr, slave_appl_ptr, transfer);
+		size -= transfer;
+		if (! size)
+			break;
+		slave_appl_ptr += transfer;
+		slave_appl_ptr %= dmix->slave_buffer_size;
+		appl_ptr += transfer;
+		appl_ptr %= pcm->buffer_size;
+	}
+	dmix->last_appl_ptr -= frames;
+	dmix->last_appl_ptr %= pcm->boundary;
+	dmix->slave_appl_ptr -= frames;
+	dmix->slave_appl_ptr %= dmix->slave_boundary;
+	dmix_up_sem(dmix);
+
 	snd_pcm_mmap_appl_backward(pcm, frames);
-	return frames;
-#else
-	return -EIO;
-#endif
+
+	return result + frames;
+}
+
+static snd_pcm_sframes_t snd_pcm_dmix_forwardable(snd_pcm_t *pcm)
+{
+	return snd_pcm_mmap_avail(pcm);
 }
 
 static snd_pcm_sframes_t snd_pcm_dmix_forward(snd_pcm_t *pcm, snd_pcm_uframes_t frames)
@@ -652,9 +769,10 @@ static int snd_pcm_dmix_close(snd_pcm_t *pcm)
  	if (dmix->client)
  		snd_pcm_direct_client_discard(dmix);
  	shm_sum_discard(dmix);
-	if (snd_pcm_direct_shm_discard(dmix))
-		snd_pcm_direct_semaphore_discard(dmix);
-	else
+	if (snd_pcm_direct_shm_discard(dmix)) {
+		if (snd_pcm_direct_semaphore_discard(dmix))
+			snd_pcm_direct_semaphore_up(dmix, DIRECT_IPC_SEM_CLIENT);
+	} else
 		snd_pcm_direct_semaphore_up(dmix, DIRECT_IPC_SEM_CLIENT);
 	free(dmix->bindings);
 	pcm->private_data = NULL;
@@ -708,6 +826,27 @@ static snd_pcm_sframes_t snd_pcm_dmix_avail_update(snd_pcm_t *pcm)
 	return snd_pcm_mmap_playback_avail(pcm);
 }
 
+static int snd_pcm_dmix_htimestamp(snd_pcm_t *pcm,
+				   snd_pcm_uframes_t *avail,
+				   snd_htimestamp_t *tstamp)
+{
+	snd_pcm_direct_t *dmix = pcm->private_data;
+	snd_pcm_uframes_t avail1;
+	int ok = 0;
+	
+	while (1) {
+		if (dmix->state == SND_PCM_STATE_RUNNING ||
+		    dmix->state == SND_PCM_STATE_DRAINING)
+			snd_pcm_dmix_sync_ptr(pcm);
+		avail1 = snd_pcm_mmap_playback_avail(pcm);
+		if (ok && *avail == avail1)
+			break;
+		*avail = avail1;
+		*tstamp = snd_pcm_hw_fast_tstamp(pcm);
+	}
+	return 0;
+}
+
 static int snd_pcm_dmix_poll_revents(snd_pcm_t *pcm, struct pollfd *pfds, unsigned int nfds, unsigned short *revents)
 {
 	snd_pcm_direct_t *dmix = pcm->private_data;
@@ -756,7 +895,9 @@ static snd_pcm_fast_ops_t snd_pcm_dmix_fast_ops = {
 	.drop = snd_pcm_dmix_drop,
 	.drain = snd_pcm_dmix_drain,
 	.pause = snd_pcm_dmix_pause,
+	.rewindable = snd_pcm_dmix_rewindable,
 	.rewind = snd_pcm_dmix_rewind,
+	.forwardable = snd_pcm_dmix_forwardable,
 	.forward = snd_pcm_dmix_forward,
 	.resume = snd_pcm_direct_resume,
 	.link = NULL,
@@ -768,6 +909,7 @@ static snd_pcm_fast_ops_t snd_pcm_dmix_fast_ops = {
 	.readn = snd_pcm_dmix_readn,
 	.avail_update = snd_pcm_dmix_avail_update,
 	.mmap_commit = snd_pcm_dmix_mmap_commit,
+	.htimestamp = snd_pcm_dmix_htimestamp,
 	.poll_descriptors = NULL,
 	.poll_descriptors_count = NULL,
 	.poll_revents = snd_pcm_dmix_poll_revents,
@@ -1137,7 +1279,7 @@ int _snd_pcm_dmix_open(snd_pcm_t **pcmp, const char *name,
 	params.periods = 3;
 
 	err = snd_pcm_slave_conf(root, dopen.slave, &sconf, 8,
-				 SND_PCM_HW_PARAM_FORMAT, 0, &params.format,
+				 SND_PCM_HW_PARAM_FORMAT, SCONF_UNCHANGED, &params.format,
 				 SND_PCM_HW_PARAM_RATE, 0, &params.rate,
 				 SND_PCM_HW_PARAM_CHANNELS, 0, &params.channels,
 				 SND_PCM_HW_PARAM_PERIOD_TIME, 0, &params.period_time,
@@ -1152,8 +1294,10 @@ int _snd_pcm_dmix_open(snd_pcm_t **pcmp, const char *name,
 	if (psize == -1 && params.period_time == -1)
 		params.period_time = 125000;    /* 0.125 seconds */
 
-	/* sorry, limited features */
-	if (! (dmix_supported_format & (1ULL << params.format))) {
+	if (params.format == -2)
+		params.format = SND_PCM_FORMAT_UNKNOWN;
+	else if (!(dmix_supported_format & (1ULL << params.format))) {
+		/* sorry, limited features */
 		SNDERR("Unsupported format");
 		snd_config_delete(sconf);
 		return -EINVAL;
