@@ -194,11 +194,8 @@ static int snd_pcm_dshare_sync_ptr(snd_pcm_t *pcm)
 	if (avail > dshare->avail_max)
 		dshare->avail_max = avail;
 	if (avail >= pcm->stop_threshold) {
-		struct timeval tv;
 		snd_timer_stop(dshare->timer);
-		gettimeofday(&tv, 0);
-		dshare->trigger_tstamp.tv_sec = tv.tv_sec;
-		dshare->trigger_tstamp.tv_nsec = tv.tv_usec * 1000L;
+		gettimestamp(&dshare->trigger_tstamp, pcm->monotonic);
 		if (dshare->state == SND_PCM_STATE_RUNNING) {
 			dshare->state = SND_PCM_STATE_XRUN;
 			return -EPIPE;
@@ -229,7 +226,7 @@ static int snd_pcm_dshare_status(snd_pcm_t *pcm, snd_pcm_status_t * status)
 	memset(status, 0, sizeof(*status));
 	status->state = snd_pcm_state(dshare->spcm);
 	status->trigger_tstamp = dshare->trigger_tstamp;
-	status->tstamp = snd_pcm_hw_fast_tstamp(dshare->spcm);
+	gettimestamp(&status->tstamp, pcm->monotonic);
 	status->avail = snd_pcm_mmap_playback_avail(pcm);
 	status->avail_max = status->avail > dshare->avail_max ? status->avail : dshare->avail_max;
 	dshare->avail_max = 0;
@@ -335,7 +332,6 @@ static int snd_pcm_dshare_start(snd_pcm_t *pcm)
 {
 	snd_pcm_direct_t *dshare = pcm->private_data;
 	snd_pcm_sframes_t avail;
-	struct timeval tv;
 	int err;
 	
 	if (dshare->state != SND_PCM_STATE_PREPARED)
@@ -350,9 +346,7 @@ static int snd_pcm_dshare_start(snd_pcm_t *pcm)
 			return err;
 		snd_pcm_dshare_sync_area(pcm);
 	}
-	gettimeofday(&tv, 0);
-	dshare->trigger_tstamp.tv_sec = tv.tv_sec;
-	dshare->trigger_tstamp.tv_nsec = tv.tv_usec * 1000L;
+	gettimestamp(&dshare->trigger_tstamp, pcm->monotonic);
 	return 0;
 }
 
@@ -416,15 +410,27 @@ static int snd_pcm_dshare_pause(snd_pcm_t *pcm ATTRIBUTE_UNUSED, int enable ATTR
 	return -EIO;
 }
 
-static snd_pcm_sframes_t snd_pcm_dshare_rewind(snd_pcm_t *pcm ATTRIBUTE_UNUSED, snd_pcm_uframes_t frames ATTRIBUTE_UNUSED)
+static snd_pcm_sframes_t snd_pcm_dshare_rewindable(snd_pcm_t *pcm)
 {
-#if 0
-	/* FIXME: substract samples from the mix ring buffer, too? */
+	return snd_pcm_mmap_playback_hw_avail(pcm);
+}
+
+static snd_pcm_sframes_t snd_pcm_dshare_rewind(snd_pcm_t *pcm, snd_pcm_uframes_t frames)
+{
+	snd_pcm_sframes_t avail;
+
+	avail = snd_pcm_mmap_playback_hw_avail(pcm);
+	if (avail < 0)
+		return 0;
+	if (frames > (snd_pcm_uframes_t)avail)
+		frames = avail;
 	snd_pcm_mmap_appl_backward(pcm, frames);
 	return frames;
-#else
-	return -EIO;
-#endif
+}
+
+static snd_pcm_sframes_t snd_pcm_dshare_forwardable(snd_pcm_t *pcm)
+{
+	return snd_pcm_mmap_playback_avail(pcm);
 }
 
 static snd_pcm_sframes_t snd_pcm_dshare_forward(snd_pcm_t *pcm, snd_pcm_uframes_t frames)
@@ -520,6 +526,27 @@ static snd_pcm_sframes_t snd_pcm_dshare_avail_update(snd_pcm_t *pcm)
 	return snd_pcm_mmap_playback_avail(pcm);
 }
 
+static int snd_pcm_dshare_htimestamp(snd_pcm_t *pcm,
+				     snd_pcm_uframes_t *avail,
+				     snd_htimestamp_t *tstamp)
+{
+	snd_pcm_direct_t *dshare = pcm->private_data;
+	snd_pcm_uframes_t avail1;
+	int ok = 0;
+	
+	while (1) {
+		if (dshare->state == SND_PCM_STATE_RUNNING ||
+		    dshare->state == SND_PCM_STATE_DRAINING)
+			snd_pcm_dshare_sync_ptr(pcm);
+		avail1 = snd_pcm_mmap_playback_avail(pcm);
+		if (ok && *avail == avail1)
+			break;
+		*avail = avail1;
+		*tstamp = snd_pcm_hw_fast_tstamp(pcm);
+	}
+	return 0;
+}
+
 static void snd_pcm_dshare_dump(snd_pcm_t *pcm, snd_output_t *out)
 {
 	snd_pcm_direct_t *dshare = pcm->private_data;
@@ -559,7 +586,9 @@ static snd_pcm_fast_ops_t snd_pcm_dshare_fast_ops = {
 	.drop = snd_pcm_dshare_drop,
 	.drain = snd_pcm_dshare_drain,
 	.pause = snd_pcm_dshare_pause,
+	.rewindable = snd_pcm_dshare_rewindable,
 	.rewind = snd_pcm_dshare_rewind,
+	.forwardable = snd_pcm_dshare_forwardable,
 	.forward = snd_pcm_dshare_forward,
 	.resume = snd_pcm_direct_resume,
 	.link = NULL,
@@ -571,6 +600,7 @@ static snd_pcm_fast_ops_t snd_pcm_dshare_fast_ops = {
 	.readn = snd_pcm_dshare_readn,
 	.avail_update = snd_pcm_dshare_avail_update,
 	.mmap_commit = snd_pcm_dshare_mmap_commit,
+	.htimestamp = snd_pcm_dshare_htimestamp,
 	.poll_descriptors = NULL,
 	.poll_descriptors_count = NULL,
 	.poll_revents = snd_pcm_direct_poll_revents,
@@ -878,7 +908,7 @@ int _snd_pcm_dshare_open(snd_pcm_t **pcmp, const char *name,
 	bsize = psize = -1;
 	params.periods = 3;
 	err = snd_pcm_slave_conf(root, dopen.slave, &sconf, 8,
-				 SND_PCM_HW_PARAM_FORMAT, 0, &params.format,
+				 SND_PCM_HW_PARAM_FORMAT, SCONF_UNCHANGED, &params.format,
 				 SND_PCM_HW_PARAM_RATE, 0, &params.rate,
 				 SND_PCM_HW_PARAM_CHANNELS, 0, &params.channels,
 				 SND_PCM_HW_PARAM_PERIOD_TIME, 0, &params.period_time,
@@ -893,6 +923,9 @@ int _snd_pcm_dshare_open(snd_pcm_t **pcmp, const char *name,
 	if (psize == -1 && params.period_time == -1)
 		params.period_time = 125000;	/* 0.125 seconds */
 
+	if (params.format == -2)
+		params.format = SND_PCM_FORMAT_UNKNOWN;
+
 	params.period_size = psize;
 	params.buffer_size = bsize;
 
