@@ -62,11 +62,9 @@ static int snd_pcm_mmap_emul_hw_refine(snd_pcm_t *pcm,
 	snd_mask_none(&mask);
 	err = snd_pcm_hw_refine(map->gen.slave, params);
 	if (err < 0) {
-		/* try to use RW_* */
 		snd_pcm_hw_params_t new = *params;
 
-		if (!(params->rmask & (1<<SND_PCM_HW_PARAM_ACCESS)))
-			return err;
+		/* try to use RW_* */
 		if (snd_pcm_access_mask_test(&oldmask,
 					     SND_PCM_ACCESS_MMAP_INTERLEAVED) &&
 		    !snd_pcm_access_mask_test(&oldmask,
@@ -156,7 +154,7 @@ static int snd_pcm_mmap_emul_hw_params(snd_pcm_t *pcm,
 	snd_pcm_hw_params_t old = *params;
 	snd_pcm_access_t access;
 	snd_pcm_access_mask_t oldmask;
-	const snd_mask_t *pmask;
+	snd_pcm_access_mask_t *pmask;
 	int err;
 
 	err = _snd_pcm_hw_params(map->gen.slave, params);
@@ -166,21 +164,20 @@ static int snd_pcm_mmap_emul_hw_params(snd_pcm_t *pcm,
 	}
 
 	*params = old;
-	pmask = snd_pcm_hw_param_get_mask(params, SND_PCM_HW_PARAM_ACCESS);
-	oldmask = *(snd_pcm_access_mask_t *)pmask;
+	pmask = (snd_pcm_access_mask_t *)snd_pcm_hw_param_get_mask(params, SND_PCM_HW_PARAM_ACCESS);
+	oldmask = *pmask;
 	if (INTERNAL(snd_pcm_hw_params_get_access)(params, &access) < 0)
 		goto _err;
 	switch (access) {
 	case SND_PCM_ACCESS_MMAP_INTERLEAVED:
-		snd_pcm_access_mask_reset((snd_pcm_access_mask_t *)pmask,
+		snd_pcm_access_mask_reset(pmask,
 					  SND_PCM_ACCESS_MMAP_INTERLEAVED);
-		snd_pcm_access_mask_set((snd_pcm_access_mask_t *)pmask,
-					SND_PCM_ACCESS_RW_INTERLEAVED);
+		snd_pcm_access_mask_set(pmask, SND_PCM_ACCESS_RW_INTERLEAVED);
 		break;
 	case SND_PCM_ACCESS_MMAP_NONINTERLEAVED:
-		snd_pcm_access_mask_reset((snd_pcm_access_mask_t *)pmask,
+		snd_pcm_access_mask_reset(pmask,
 					  SND_PCM_ACCESS_MMAP_NONINTERLEAVED);
-		snd_pcm_access_mask_set((snd_pcm_access_mask_t *)pmask,
+		snd_pcm_access_mask_set(pmask,
 					SND_PCM_ACCESS_RW_NONINTERLEAVED);
 		break;
 	default:
@@ -191,7 +188,7 @@ static int snd_pcm_mmap_emul_hw_params(snd_pcm_t *pcm,
 		goto _err;
 
 	/* need to back the access type to relieve apps */
-	*(snd_pcm_access_mask_t *)pmask = oldmask;
+	*pmask = oldmask;
 
 	/* OK, we do fake */
 	map->mmap_emul = 1;
@@ -260,7 +257,6 @@ sync_slave_write(snd_pcm_t *pcm)
 	size = map->appl_ptr - *slave->appl.ptr;
 	if (size < 0)
 		size += pcm->boundary;
-	size -= size % pcm->xfer_align;
 	if (!size)
 		return 0;
 	offset = *slave->appl.ptr % pcm->buffer_size;
@@ -279,7 +275,6 @@ sync_slave_read(snd_pcm_t *pcm)
 	size = *slave->hw.ptr - map->hw_ptr;
 	if (size < 0)
 		size += pcm->boundary;
-	size -= size % pcm->xfer_align;
 	if (!size)
 		return 0;
 	offset = map->hw_ptr % pcm->buffer_size;
@@ -360,7 +355,9 @@ static snd_pcm_fast_ops_t snd_pcm_mmap_emul_fast_ops = {
 	.drop = snd_pcm_generic_drop,
 	.drain = snd_pcm_generic_drain,
 	.pause = snd_pcm_generic_pause,
+	.rewindable = snd_pcm_generic_rewindable,
 	.rewind = snd_pcm_mmap_emul_rewind,
+	.forwardable = snd_pcm_generic_forwardable,
 	.forward = snd_pcm_mmap_emul_forward,
 	.resume = snd_pcm_generic_resume,
 	.link = snd_pcm_generic_link,
@@ -372,13 +369,15 @@ static snd_pcm_fast_ops_t snd_pcm_mmap_emul_fast_ops = {
 	.readn = snd_pcm_generic_readn,
 	.avail_update = snd_pcm_mmap_emul_avail_update,
 	.mmap_commit = snd_pcm_mmap_emul_mmap_commit,
+	.htimestamp = snd_pcm_generic_htimestamp,
 	.poll_descriptors = snd_pcm_generic_poll_descriptors,
 	.poll_descriptors_count = snd_pcm_generic_poll_descriptors_count,
 	.poll_revents = snd_pcm_generic_poll_revents,
 };
 
-static int snd_pcm_mmap_emul_open(snd_pcm_t **pcmp, const char *name,
-				  snd_pcm_t *slave, int close_slave)
+#ifndef DOC_HIDDEN
+int __snd_pcm_mmap_emul_open(snd_pcm_t **pcmp, const char *name,
+			     snd_pcm_t *slave, int close_slave)
 {
 	snd_pcm_t *pcm;
 	mmap_emul_t *map;
@@ -401,12 +400,14 @@ static int snd_pcm_mmap_emul_open(snd_pcm_t **pcmp, const char *name,
 	pcm->private_data = map;
 	pcm->poll_fd = slave->poll_fd;
 	pcm->poll_events = slave->poll_events;
+	pcm->monotonic = slave->monotonic;
 	snd_pcm_set_hw_ptr(pcm, &map->hw_ptr, -1, 0);
 	snd_pcm_set_appl_ptr(pcm, &map->appl_ptr, -1, 0);
 	*pcmp = pcm;
 
 	return 0;
 }
+#endif
 
 /*! \page pcm_plugins
 
@@ -474,7 +475,7 @@ int _snd_pcm_mmap_emul_open(snd_pcm_t **pcmp, const char *name,
 	snd_config_delete(sconf);
 	if (err < 0)
 		return err;
-	err = snd_pcm_mmap_emul_open(pcmp, name, spcm, 1);
+	err = __snd_pcm_mmap_emul_open(pcmp, name, spcm, 1);
 	if (err < 0)
 		snd_pcm_close(spcm);
 	return err;
