@@ -78,31 +78,14 @@ static inline int hw_is_interval(snd_pcm_hw_param_t var)
 		var <= SND_PCM_HW_PARAM_LAST_INTERVAL;
 }
 
-static inline snd_mask_t *hw_param_mask(snd_pcm_hw_params_t *params,
-					snd_pcm_hw_param_t var)
-{
-	assert(hw_is_mask(var));
-	return (snd_mask_t*)&params->masks[var - SND_PCM_HW_PARAM_FIRST_MASK];
-}
+#define hw_param_mask(params,var) \
+	&((params)->masks[(var) - SND_PCM_HW_PARAM_FIRST_MASK])
 
-static inline snd_interval_t *hw_param_interval(snd_pcm_hw_params_t *params,
-						snd_pcm_hw_param_t var)
-{
-	assert(hw_is_interval(var));
-	return &params->intervals[var - SND_PCM_HW_PARAM_FIRST_INTERVAL];
-}
+#define hw_param_interval(params,var) \
+	&((params)->intervals[(var) - SND_PCM_HW_PARAM_FIRST_INTERVAL])
 
-static inline const snd_mask_t *hw_param_mask_c(const snd_pcm_hw_params_t *params,
-						snd_pcm_hw_param_t var)
-{
-	return (const snd_mask_t *)hw_param_mask((snd_pcm_hw_params_t*) params, var);
-}
-
-static inline const snd_interval_t *hw_param_interval_c(const snd_pcm_hw_params_t *params,
-							snd_pcm_hw_param_t var)
-{
-	return (const snd_interval_t *)hw_param_interval((snd_pcm_hw_params_t*) params, var);
-}
+#define hw_param_mask_c hw_param_mask
+#define hw_param_interval_c hw_param_interval
 
 static void _snd_pcm_hw_param_any(snd_pcm_hw_params_t *params, snd_pcm_hw_param_t var)
 {
@@ -834,7 +817,7 @@ int snd_pcm_hw_param_set_near(snd_pcm_t *pcm, snd_pcm_hw_params_t *params,
 	int err;
 	unsigned int best = *val, saved_min;
 	int last = 0;
-	int min, max;
+	unsigned int min, max;
 	int mindir, maxdir;
 	int valdir = dir ? *dir : 0;
 	snd_interval_t *i;
@@ -865,9 +848,7 @@ int snd_pcm_hw_param_set_near(snd_pcm_t *pcm, snd_pcm_hw_params_t *params,
 	
 	if (err >= 0) {
 		snd_pcm_hw_params_t params1;
-		if (max < 0)
-			goto _end;
-		if ((unsigned int)min == saved_min && mindir == valdir)
+		if (min == saved_min && mindir == valdir)
 			goto _end;
 		params1 = save;
 		err = snd_pcm_hw_param_set_max(pcm, &params1, SND_CHANGE, var, &max, &maxdir);
@@ -2253,7 +2234,7 @@ static int snd_pcm_sw_params_default(snd_pcm_t *pcm, snd_pcm_sw_params_t *params
 	params->period_step = 1;
 	params->sleep_min = 0;
 	params->avail_min = pcm->period_size;
-	params->xfer_align = pcm->period_size;
+	params->xfer_align = 1;
 	params->start_threshold = 1;
 	params->stop_threshold = pcm->buffer_size;
 	params->silence_threshold = 0;
@@ -2328,7 +2309,6 @@ int _snd_pcm_hw_params(snd_pcm_t *pcm, snd_pcm_hw_params_t *params)
 	INTERNAL(snd_pcm_hw_params_get_period_time)(params, &pcm->period_time, 0);
 	INTERNAL(snd_pcm_hw_params_get_period_size)(params, &pcm->period_size, 0);
 	INTERNAL(snd_pcm_hw_params_get_buffer_size)(params, &pcm->buffer_size);
-	INTERNAL(snd_pcm_hw_params_get_tick_time)(params, &pcm->tick_time, 0);
 	pcm->sample_bits = snd_pcm_format_physical_width(pcm->format);
 	pcm->frame_bits = pcm->sample_bits * pcm->channels;
 	fb = pcm->frame_bits;
